%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MAIN SCRIPT                                                          % 
%                                                                      % 
% 1) Simulate Data from Random Coefficients Logit Demand model         %
% 2) Estimate model using MPEC                                         %
%                                                                      % 
% source: Dube, Fox and Su (2012)                                      %    
% Code Revised: January 2012                                           %    
% Code Modified by P. Ketz: December 2015                              %
% Code Revised: May 2017                                               %
%                                                                      %
% COMMENT: The main modification consists in estimating the model with %
% respect to variances rather than standard deviations. In addition,   %
% sparse grid integration as suggested by Skrainka and Judd (2011) and %
% the use of optimal instruments as suggested by Reynaert and Verboven %
% (2014) are implemented. Sparse grid integration is performed using   %
% the function nwspgr.m provided by Heiss and Winschel (2008). Lastly, %
% the estimator proposed in "Subvector Inference When the True         % 
% Parameter Vector is near the Boundary" and the modified J-statistic  %
% proposed in "Testing Overidentifying Restrictions when the True      %
% Parameter Vector Is near or at the Boundary of the Parameter Space"  %
% are implemented.                                                     %                                
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

%%%%%%%%%%%%  
% SETTINGS %
%%%%%%%%%%%%
global share W x x_rc IV
global marketForProducts sharesum oo
global T prods K K_rc tol_inner 
global prodsMarket marketStarts marketEnds numProdsTotal
global points weights num_points

options_fmincon = optimset('MaxIter',100,'Algorithm','interior-point',...
    'Display','off','GradConstr','on','GradObj','on','TolCon',1E-6,...
    'TolFun',1E-6,'TolX',1E-6,'Hessian','user-supplied',...
    'HessFcn',@GMMMPEC_hess); 

rng(15);
   
prods = 10;                                                 % maximum # products
T = 25;                                                     % # Markets
starts = 2;                                                 % # random start values to check during estimation
tol_inner = 1.e-14;                                         % NFP inner loop tolerance, for starting values here

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Number of products per market        %
% Discrete uniform between 1 and prods %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
prodsMarket = prods * ones(T,1); 

%% Alternative: 
% % We can let the number of products vary across markets:
% randi([1 prods],T,1);  
%% End of alternative.

%%
% Use two dimensional matrices for speed reasons (vectorization)
% Market indices tell us the first and last spots of records for each
% market in list of all products
marketStarts = zeros(T,1);
marketEnds = zeros(T,1);
marketStarts(1) = 1;
marketEnds(1) = prodsMarket(1);
for t=2:T
    marketStarts(t) = marketEnds(t-1) + 1;
    marketEnds(t) = marketStarts(t) + prodsMarket(t) - 1;
end;
numProdsTotal = marketEnds(T);                              % Total number of products in all markets

%%%%%%%%%%%%%%%%%%%%%%%%%
% TRUE PARAMETER VALUES %
%%%%%%%%%%%%%%%%%%%%%%%%%
beta = [-2 2 2]'; 
K = size(beta,1);
Pi_1 = [0.7 0.7]';
Pi_2 = [3 3 3]';
Pi = [Pi_1;Pi_2];
L = size(Pi,1);     
sigmarc = diag([0]);                                        % True standard deviations, NOT variances  
K_rc = size(sigmarc,2);
rho = 0.7;
Omega = [1 rho;rho 1];

[points,weights] = nwspgr('GQN', K_rc, 7);
points = points';
weights = weights';

num_points = size(points,2);
rc = sigmarc*points;

thetaTRUE = [beta;diag(sigmarc).^2];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Create indices to speed share calculations %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
oo = ones(1,size(points,2));                                % index for use in simulation of shares

sharesum = sparse(zeros(T,numProdsTotal));                  % used to create denominators in logit predicted shares (i.e. sums numerators)
for t=1:T
    sharesum(t,marketStarts(t):marketEnds(t)) = 1;
end

marketForProducts = zeros(numProdsTotal,1);                 % market indices for each product
% used to expand one logit denominator per consumer to one for each product
for t=1:T
    marketForProducts(marketStarts(t):marketEnds(t)) = t;
end

%%%%%%%%%%%%%%%%%
% SIMULATE DATA %
%%%%%%%%%%%%%%%%%
errors = randn(numProdsTotal,2)*chol(Omega);                % draw shocks
xi = errors(:,1);                                           % structural error
omega = errors(:,2);                                        
w = ones(numProdsTotal,1) + rand(numProdsTotal,1);          % draw exogenous regressor
z = rand(numProdsTotal,3);                                  % draw instruments
p = [ones(numProdsTotal,1), w, z] * Pi + omega;             % compute endogenous variable  
x_nrc = [p ones(numProdsTotal,1)];                          % all regressors without random coefficients
x_rc = w;                                                   % all regressors with random coefficients 
x = [x_nrc, x_rc];                                          % all regressors

% mksahresg replaces mksharesim because we rely on sparse grid integration
% rather than Monte Carlo integration
[share,nopurch] = mksharesg(beta,xi,rc);                    

y = log(share) - log(nopurch(marketForProducts,:));         % log-odds ratio for no RC logit estimation
IV = [ones(numProdsTotal,1) w z];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2SLS ESTIMATION OF HOMOGENEOUS LOGIT MODEL %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
W = inv(IV'*IV);                       
ZX = IV'*x;
ZY = IV'*y;
XZW = ZX'*W;
XZWZX = XZW*ZX;
XZWZY = XZW*ZY;
beta2sls = XZWZX\XZWZY;

expmeanval0 = exp(y);                                       % starting value for mean values

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MPEC ESTIMATION OF RANDOM COEFFICIENTS LOGIT MODEL - 1st step %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
temp = abs(beta2sls(K-K_rc+1:K));
startvalues = repmat(temp,1,starts).*[ones(1,size(temp,1));rand([starts-1,size(temp',2)])]'; 

GMPEC = 1.0e20;
CPUtMPEC = 0;
FuncEvalMPEC = 0;
for reps=1:starts
    % Setting starting values
    sigma20 = startvalues(:,reps);
    delta0 = invertshares(sigma20,expmeanval0,10000);  
    ZY_temp = IV'*delta0;
    XZWZY_temp = XZW*ZY_temp;
    mu0 = XZWZX\XZWZY_temp;     
    xi0 = delta0 - x*mu0;            
    g0 = IV'*xi0; 

    x0 = [mu0; sigma20; delta0; g0];
    
    x_L = -Inf*ones(length(x0),1);                          % Lower bounds for x.
    x_L(K+1:K+K_rc,1) = zeros(K_rc,1);                      % standard deviations are nonnegative
    x_U =  Inf*ones(length(x0),1);                          % Upper bounds for x. 
    
    t1 = cputime; 

    [X, fval_rep, exitflag, output, lambda] = fmincon(@GMMMPEC_f,x0,[],[],[],[],x_L,x_U,@GMMMPEC_c,options_fmincon); 

    CPUtMPEC_rep = cputime - t1;

    muMPEC_rep = X(1:K);
    sigma2MPEC_rep = X(K+1:K+K_rc);
    deltaMPEC_rep = X(K+K_rc+1:K+K_rc+numProdsTotal);
    GMPEC_rep = fval_rep;
    INFOMPEC_rep = exitflag;

    CPUtMPEC = CPUtMPEC + CPUtMPEC_rep;    
    FuncEvalMPEC = FuncEvalMPEC + output.funcCount;

    % if using knitro: replace (INFOMPEC_rep==1 | INFOMPEC_rep==2) by INFOMPEC_rep==0
    if (GMPEC_rep < GMPEC && (INFOMPEC_rep==1 || INFOMPEC_rep==2))
        muMPEC = muMPEC_rep;
        sigma2MPEC = sigma2MPEC_rep;
        sigmaMPEC = sigma2MPEC_rep.^0.5;
        deltaMPEC = deltaMPEC_rep;
        GMPEC = GMPEC_rep;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MPEC ESTIMATION OF RANDOM COEFFICIENTS LOGIT MODEL - 2nd step %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PiOLS = (IV'*IV)\(IV'*p);
p_hat = IV * PiOLS;
delta_hat = [p_hat ones(numProdsTotal,1) w]*muMPEC;
Jac_opt = jacobian(sigma2MPEC,delta_hat,x_rc);

% Note: The "optimal" instruments, p_hat and Jac_opt, are only optimal 
% under the assumption of homoskedasticity.

IV = [ones(numProdsTotal,1) w p_hat Jac_opt];
W = eye(size(IV,2)); % The model is just identified - the choice of W doesn't matter

%% Alternative: 
% % We may, for some reason, want to use a larger set of instruments, e.g., 
% % we may want use the set of all instruments for p, z, rather than the 
% % "optimal" one, p_hat. (Then, the model is overidentified and we can 
% % implement an overidentification test, see below.) Then, we may want to 
% % use the efficient weighting matrix:
% IV = [ones(numProdsTotal,1) w z Jac_opt];
% resid = deltaMPEC - x*muMPEC;  
% covg = zeros(size(IV,2));
% for t=1:T,
%     covg = covg + IV(marketStarts(t):marketEnds(t),:)'...
%         *resid(marketStarts(t):marketEnds(t))...
%         *resid(marketStarts(t):marketEnds(t))'...
%         *IV(marketStarts(t):marketEnds(t),:);
% end
% W = inv(covg);
%% End of alternative.

ZX = IV'*x;
XZW = ZX'*W;
XZWZX = XZW*ZX;

%%
temp = sigma2MPEC;
startvalues = repmat(temp,1,starts).*[ones(1,size(temp,1));rand([starts-1,size(temp',2)])]'; 

GMPEC = 1.0e20;
for reps=1:starts
    % Setting starting values
    sigma20 = startvalues(:,reps);
    delta0 = invertshares(sigma20,expmeanval0,10000);  
    ZY_temp = IV'*delta0;
    XZWZY_temp = XZW*ZY_temp;
    mu0 = XZWZX\XZWZY_temp;     
    xi0 = delta0 - x*mu0;            
    g0 = IV'*xi0; 

    x0 = [mu0; sigma20; delta0; g0];
    
    x_L = -Inf*ones(length(x0),1);                          % Lower bounds for x.
    x_L(K+1:K+K_rc,1) = zeros(K_rc,1);                      % standard deviations are nonnegative
    x_U =  Inf*ones(length(x0),1);                          % Upper bounds for x. 
    
    t1 = cputime; 

    [X, fval_rep, exitflag, output, lambda] = fmincon(@GMMMPEC_f,x0,[],[],[],[],x_L,x_U,@GMMMPEC_c,options_fmincon); 

    CPUtMPEC_rep = cputime - t1;

    muMPEC_rep = X(1:K);
    sigma2MPEC_rep = X(K+1:K+K_rc);
    deltaMPEC_rep = X(K+K_rc+1:K+K_rc+numProdsTotal);
    GMPEC_rep = fval_rep;
    INFOMPEC_rep = exitflag;

    CPUtMPEC = CPUtMPEC + CPUtMPEC_rep;    
    FuncEvalMPEC = FuncEvalMPEC + output.funcCount;

    % if using knitro: replace (INFOMPEC_rep==1 | INFOMPEC_rep==2) by INFOMPEC_rep==0
    if (GMPEC_rep < GMPEC && (INFOMPEC_rep==1 || INFOMPEC_rep==2))
        muMPEC = muMPEC_rep;
        sigma2MPEC = sigma2MPEC_rep;
        sigmaMPEC = sigma2MPEC_rep.^0.5;
        deltaMPEC = deltaMPEC_rep;
        GMPEC = GMPEC_rep;
    end
end

thetaMPEC = [muMPEC;sigma2MPEC];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% COVARIANCE MATRIX FOR MPEC STRUCTURAL PARAMETERS %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
resid = deltaMPEC - x*muMPEC;  
covg = zeros(size(IV,2));
for t=1:T,
    covg = covg + IV(marketStarts(t):marketEnds(t),:)'...
        *resid(marketStarts(t):marketEnds(t))...
        *resid(marketStarts(t):marketEnds(t))'...
        *IV(marketStarts(t):marketEnds(t),:);
end
Ddelta = jacobian(sigma2MPEC,deltaMPEC,x_rc);      
Dg = [-x Ddelta]'*IV;

covMPEC = inv(Dg*W*Dg')*Dg*W*covg*W*Dg'*inv(Dg*W*Dg');

%% Alternative: 
% % The following estimator can be used. If the model is just
% % identified, the two expressions are identical. Furthermore, if the model
% % is overidentified and the efficient weighting matrix is employed, the two
% % estimators are asymptotically equivalent.
% covMPEC = inv(Dg*inv(covg)*Dg');
%% End of Alternative.

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ASYMPTOTICALLY NORMAL ESTIMATOR %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if min(sigma2MPEC) < 0.001
    thetaMPECnormal = thetaMPEC - (Dg*W*Dg')\(Dg*W*IV'*resid);
else
    thetaMPECnormal = thetaMPEC;
end

%% If model is overidentified:
%%%%%%%%%%%%%%%%%%%%%%%%
% MODIFIED J-STATISTIC %
%%%%%%%%%%%%%%%%%%%%%%%%
% temp = thetaMPECnormal-thetaMPEC;
% MJ = GMPEC + 2*(Dg*W*IV'*resid)'*temp + temp'*(Dg*W*Dg')*temp;
%%

save BLP_MPEC thetaTRUE thetaMPEC thetaMPECnormal covMPEC CPUtMPEC FuncEvalMPEC

